/**
	@file libtfshm.h
	@brief 簡易共有メモリライブラリ
	@author Atsushi WATANEBE <http://www.roboken.esys.tsukuba.ac.jp/~atusi_w/>
 */

#ifndef __LIBTFSHM_H__
#define __LIBTFSHM_H__

#ifdef __cplusplus
extern "C"
{
#endif

#include <fcntl.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdint.h>

#define LIBTFSHM_NAME_LENGTH	512
#define LIBTFSHM_DEFAULT_PATH	("/dev/shm")

	typedef struct TFSHM_CONTROL
	{
		int64_t pid;
		int64_t size;
		struct timeval timestamp;
		char reserved[128];
	} tfshm_control;

	typedef struct TFSHM
	{
		int64_t data;
		int64_t event;
		int64_t param;
		char name[LIBTFSHM_NAME_LENGTH];
		char path[LIBTFSHM_NAME_LENGTH];
		int64_t did;
		struct timeval timestamp;
	} tfshm;

	tfshm *tfshm_open( char *name );
	tfshm *tfshm_openex( char *name, char *path );
	tfshm *tfshm_create( char *name );
	tfshm *tfshm_createex( char *name, char *path );
	size_t tfshm_read( tfshm * port, void *data, size_t size );
	size_t tfshm_read_nonblock( tfshm * port, void *data, size_t size );
	size_t tfshm_readex( tfshm * port, void **data, size_t size, int autoalloc, int block, tfshm_control * ctl );
	size_t tfshm_write( tfshm * port, void *data, size_t size );
	size_t tfshm_writeex( tfshm * port, void *data, tfshm_control * ctl );
	size_t tfshm_getparam( tfshm * port, void *data, size_t size );
	size_t tfshm_putparam( tfshm * port, void *data, size_t size );
	void tfshm_close( tfshm * port );

#ifdef __cplusplus
}
#endif

#endif
