
/**
	@file readwriten.c
	@brief 低レベルIOインターフェース
	@author Atsushi WATANEBE <atusi_w@roboken.esys.tsukuba.ac.jp> <atusi_w@doramanjyu.com> 
 */

#define _XOPEN_SOURCE
#define _BSD_SOURCE
#define _GNU_SOURCE

#include <sys/time.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/file.h>
#include <sys/types.h>
#include <termios.h>
#include <errno.h>
#include "readwrite-nbyte.h"
	// #define DEBUG
	/* 
	 * @brief Read n byte @param aIo IO number @param *apBuf Pointer to buffer @param aNBytes
	 @return failed: 0,
	 * succeeded: 1 */
int readn( int aIo, void *apBuf, int aNBytes )
{

	// Loop variable
	int i, j;

	j = 0;
	for ( i = 0; i < aNBytes; )
	{
		int rs;

		rs = read( aIo, ( char * )apBuf + i, aNBytes - i );
		if( rs == -1 )
		{
			switch ( errno )
			{
			case EBADF:
				// fprintf(stderr,"EBADF\n");
				break;
			case EINVAL:
				// fprintf(stderr,"EINVAL\n");
				break;
			case EIO:
				// fprintf(stderr,"EIO\n");
				return -1;
				break;
			case EFAULT:
				// fprintf(stderr,"EFAULT\n");
				break;
			case EINTR:
				// fprintf(stderr,"EINTR\n");
				break;
			}
			j++;
			if( j < 100 )
			{
				usleep( 200 );
				continue;
			}
			return -1;
		}
		if( rs == 0 )
		{
			j++;
			if( j < 100 )
			{
				usleep( 200 );
				continue;
			}
			return 0;
		}
		i += rs;
	}
	return 1;
}

	/* 
	 * @brief Write n byte @param aIo IO number @param *apBuf Pointer to buffer @param aNBytes
	 @return failed: 0,
	 * succeeded: 1 */
int writen( int aIo, void *apBuf, int aNBytes )
{

	// Loop variable
	int i, j;

	j = 0;
	for ( i = 0; i < aNBytes; )
	{
		int rs;

		rs = write( aIo, ( char * )apBuf + i, aNBytes - i );
		if( rs == -1 )
		{
			switch ( errno )
			{
			case EBADF:
				// fprintf(stderr,"WEBADF\n");
				break;
			case EINVAL:
				// fprintf(stderr,"WEINVAL\n");
				break;
			case EIO:
				// fprintf(stderr,"WEIO\n");
				return -1;
				break;
			case EFAULT:
				// fprintf(stderr,"WEFAULT\n");
				break;
			case EINTR:
				// fprintf(stderr,"WEINTR\n");
				break;
			}
			j++;
			if( j < 100 )
			{
				usleep( 200 );
				continue;
			}
			return -1;
		}
		if( rs == 0 )
		{
			j++;
			if( j < 100 )
			{
				usleep( 200 );
				continue;
			}
			return 0;
		}
		i += rs;
	}
	return 1;
}
